/**
   Copyright (c) 2005 Lucas Mello Schnorr 
   
   Written by: Lucas Mello Schnorr <schnorr@gmail.com>
   
   This file is part of libPaje.
    
   libPaje is free software; you can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.
   
   libPaje is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
   for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with libPaje; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
*/
#ifndef __PAJEEVENT_H
#define __PAJEEVENT_H
#include <Foundation/Foundation.h>
#include "PajeHeader.h"
#include "PajeHeaderCenter.h"

#define PAJEDEFINECONTAINERTYPE         0
#define PAJEDEFINEENTITYVALUE           1
#define PAJEDEFINEVARIABLETYPE          2
#define PAJEDEFINELINKTYPE              3
#define PAJEDEFINESTATETYPE             4
#define PAJEDEFINEEVENTTYPE             5

#define PAJEPOPSTATE                    6
#define PAJESETSTATE                    7
#define PAJEPUSHSTATE                   8

#define PAJESTARTLINK                   9
#define PAJEENDLINK                     10

#define PAJECREATECONTAINER             11
#define PAJEDESTROYCONTAINER            12

#define PAJESETVARIABLE                 13
#define PAJESUBVARIABLE                 14
#define PAJEADDVARIABLE                 15

#define PAJENEWEVENT                    16

#define PAJESETLIMITS                   17

/**
 * This is the superclass definition of a Paje event. It should not be used
 * directly. However, its methods can be acessed through subclasses instances.
 * The subclasses of this class are:<br/>
 * <list>
 * <item><strong>[PajeAddVariable]</strong></item>
 * <item><strong>[PajeCreateContainer]</strong></item>
 * <item><strong>[PajeDefineContainerType]</strong></item>
 * <item><strong>[PajeDefineEntityValue]</strong></item>
 * <item><strong>[PajeDefineEventType]</strong></item>
 * <item><strong>[PajeDefineLinkType]</strong></item>
 * <item><strong>[PajeDefineStateType]</strong></item>
 * <item><strong>[PajeDefineVariableType]</strong></item>
 * <item><strong>[PajeDestroyContainer]</strong></item>
 * <item><strong>[PajeEndLink]</strong></item>
 * <item><strong>[PajeEvent]</strong></item>
 * <item><strong>[PajePopState]</strong></item>
 * <item><strong>[PajePushState]</strong></item>
 * <item><strong>[PajeSetState]</strong></item>
 * <item><strong>[PajeSetVariable]</strong></item>
 * <item><strong>[PajeStartLink]</strong></item>
 * <item><strong>[PajeSubVariable]</strong></item>
 * </list>
 * <p>
 * When one of these subclass are instantiated, you can use the methods of
 * [PajeEvent] to
 * extend the Paje events with new fields. This new fields can be accessed
 * throught Paje Inspection Tool when visualizing the trace file. Beyond that,
 * the methods of this class offer a way of getting the printing of the event,
 * according to the order of the fields and with a properly event code.
 * </p>
 * <strong>Commonly Used Methods</strong><br/>
 * <list>
 * <item>[-setTime:], Set the time of the event</item>
 * <item>[-printWithProvider:], Returns a string with the event printed</item>
 * <item>[-addFieldNamed:withType:], Extend the event with a new field</item>
 * <item>[-setValue:forFieldNamed:], Set the value for an extended field</item>
 * </list>
 */
@interface PajeEvent : NSObject
{
	PajeHeader *header;
	NSMutableArray *values;
	NSString *time;
	BOOL timeEvent;
}
/**
 * This method returns an [PajeHeader] object, which identifies the header for 
 * the correspoding [PajeEvent]'s subclass event.
 */
- (PajeHeader *) header;


/**
 * This method initiate an object with atime. atime can be even a floating
 * point number or a long long number. If a long long number is passed as
 * parameter, the corresponding instance methods should be used after this
 * initialization. The subclasses should implement this instance method.
 */
- (id) initWithTime: (NSString *) atime;

/**
 * This method set the time of event. The atime can be a floating point or
 * a long long number.
 */
- setTime: (NSString *) atime;

/**
 * This method returns the time of the event. The return string can be
 * even a long long or floating point number, depending of previous 
 * method initialization or setTime method
 */
- (NSString *) time;

/**
 * This method returns the type of the event, which is a integer
 */
- (int) eventType;



/**
 * This method is responsable for subtracting the floating point
 * number at atime from the floating point number in instance variable time.
 */
- subtractFloatTimeByThis: (NSString *) atime;

/**
 * This method subtracts the long long number at atime from the long long
 * number in instance variable time.
 */
- subtractLongTimeByThis: (NSString *) atime;

/*
 */
//vote for deletion
//- (BOOL) addFieldNamed: (NSString *) name withType: (NSString *) type
//	withValue: (NSString *) value;


/**
 * This event returns a string with the code of the event and the values of the
 * fields. The order of appearance is according to the header of the event.
 * Normally this event is used to print the event already filled with all 
 * values. If some of the fields is not set, this events returns nil and prints
 * a message in stderr explaining what fields were not set yet.
 * The parameter <em>provider</em> must be a instance of the class
 * [PajeHeaderCenter].
 */
- (NSString *) printWithProvider: (PajeHeaderCenter *) provider;

/**
 * The subclasses of [PajeEvent] class contains fields required by the
 * Paj visualization tool. They are used to display the event information
 * in the space-time visualization. The user can add new fields to the events
 * that are already used by Paj. For example, if during the generation of a
 * [PajeStartLink] event the programmer has access to the size of a
 * communication, this size can be added to the [PajeStartLink] event and 
 * then are displayed in Paj through the inspection tool. 
 * This method adds to the header of the event the field named <em>name</em>
 * with type <em>type</em>. If already exists a field named with the
 * <em>name</em> parameter, the method returns NO, otherwise in sucess, retuns
 * YES.
 */
- (BOOL) addFieldNamed: (NSString *) name withType: (NSString *) type;

/**
 * This method sets the value of extended field name <em>name</em> to value
 * <em>value</em>. If the extended field doesn't exists, it returns NO. If the
 * extend field is present in this event and the setting was sucessfull, it
 * returns YES. <strong>Note:</strong> This method were built to be used to set the
 * value for the new extended fields added with [-addFieldNamed:withType:]. The
 * standard fields of the subclasses of [PajeEvent] have a pre-set of methods
 * that were built to help setting the fields, please use them.
 */
- (BOOL) setValue: (NSString *) value forFieldNamed: (NSString *) name;
@end

#include "PajeDefineStateType.h"
#include "PajePushState.h"
#include "PajeAddVariable.h"
#include "PajeDefineVariableType.h"
//#include "PajeSetLimits.h"
#include "PajeCreateContainer.h"
#include "PajeDestroyContainer.h"
#include "PajeSetState.h"
#include "PajeDefineContainerType.h"
#include "PajeEndLink.h"
#include "PajeSetVariable.h"
#include "PajeDefineEntityValue.h"
#include "PajeEvent.h"
#include "PajeStartLink.h"
#include "PajeDefineEventType.h"
#include "PajeNewEvent.h"
#include "PajeSubVariable.h"
#include "PajeDefineLinkType.h"
#include "PajePopState.h"
#include "PajeHeaderCenter.h"

#endif
