/**
   Copyright (c) 2005 Lucas Mello Schnorr

   Written by: Lucas Mello Schnorr <schnorr@gmail.com>
   
   This file is part of libPaje.
   
   libPaje is free software; you can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.
   
   libPaje is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
   for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with libPaje; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
*/
#ifndef __PAJE_HIERARCHY_H
#define __PAJE_HIERARCHY_H

#include <Foundation/Foundation.h>
#include "PajeEvent.h"
#include "PajeHierarchyNode.h"

/**
 * The generic visualization tool Paje needs a hierarchy of entities to
 * show the events in the Space-time window. To define a hierarchy of entities
 * in a Paje trace file, the user needs to use the following events:
 * <list>
 * <item>[PajeDefineContainerType]</item>
 * <item>[PajeDefineEntityValue]</item>
 * <item>[PajeDefineEventType]</item>
 * <item>[PajeDefineLinkType]</item>
 * <item>[PajeDefineStateType]</item>
 * <item>[PajeDefineVariableType]</item>
 * </list>
 * Normally, each of these events has an Alias field associated. The value of
 * this field can be used later in the Paje trace file to reference the entity.
 * To manage this aliases automatically, the class PajeHierarchy receive the
 * Paje hierarchy of entities from a file in Property List format and manages it
 * to convert to the appropriate Paje events. The [-initPajeHierarchyFromFile:]
 * receives the file and initilize the hierarchy in memory. When user call
 * [-hierarchy] method, he receives a list of
 * Paje events that describe the hierarchy listed in the Property List file
 * passed to the initialization method. Anytime the user needs an alias from a
 * certain type defined in this file, the user calls [-aliasToName:]. The last
 * method of this class, [-newAliasToName:], can be used to manage others
 * aliases used to mapping, for example, containers identifications to
 * containers names.
 * <br/>
 * <br/>
 * <strong>Property List's Paje Hierarchy File Format</strong><br/>
 * Here goes an example of a Paje Hierarchy File in Property List format.
 * There are five types:
 * <list>
 * <item><strong>container</strong> - mapped to [PajeDefineContainerType]</item>
 * <item><strong>state</strong> - mapped to [PajeDefineStateType]</item>
 * <item><strong>variable</strong> - mapped to [PajeDefineVariableType]</item>
 * <item><strong>link</strong> - mapped to [PajeDefineLinkType]</item>
 * <item><strong>event</strong> - mapped to [PajeDefineEventType]</item>
 * </list>
 * The "values" key in each of this types are mapped to [PajeDefineEntityValue].
 * The <strong>root</strong> type is a special keyword and are mapped to the
 * root of hierarchy of entities of Paje in a trace file.
 *
 *
 * <br/>
 * <example>
 * {
 *    type = root;
 *    children = (
 *       {
 *           type = container;
 *           name = Machine;
 *           children = (
 *               {
 *                  type = container;
 *                  name = Thread;
 *                  children = (
 *                     {
 *                        type = state;
 *                        name = "MPIState";
 *                        values = ("mpi_send", "mpi_receive", "mpi_running");
 *                     },
 *                     {
 *                        type = state;
 *                        name = "OSState";
 *                        values = (os_thread_blocked, os_thread_running);
 *                     },
 *                     {
 *                        type = event;
 *                        name = "AnEvent";
 *                        values = ("v_event1", "v_event2");
 *                     }
 *                  );
 *               },{
 *                  type = variable;
 *                  name = "CPU Utilization";
 *               }
 *           );
 *       },
 *       {
 *           type = link;
 *           name = MPICommunication;
 *           DestContainerType = "Thread";
 *           SourceContainerType = "Thread";
 *           values = ("send");
 *       }
 *   );
 * }
 * </example>
 */
@interface PajeHierarchy : NSObject
{
	NSMutableDictionary *hierarchyAlias;
	PajeHierarchyNode *hierarchy;
}
/**
 * This method receives a <em>fileName</em> pointing to a file that
 * has a hierarchy according to the format specified above. It returns
 * a new initialized object capable of being interrogated for aliases
 * and for the Paje events.
 */
- (id) initPajeHierarchyFromFile: (NSString *) fileName;

/**
 * This method receives a <em>name</em> and returns an alias to this
 * name. If the alias for this name already exists, It returns the
 * already created alias.
 */
- (NSString *) newAliasToName: (NSString *) name;

/**
 * This method receives a <em>name</em> and returns an alias to it.
 * In case the alias doesn't exist, the method returns nil.
 */
- (NSString *) aliasToName: (NSString *) name;

/**
 * This method returns an array with Paje events that describe 
 * the hierarchy of entities passed as parameter in the initialization method.
 */
- (NSMutableArray *) hierarchy;

/**
 * This method returns an array with NSString containing 
 * all the name of the entities of type <em>type</em> in the hierarchy.
 */
- (NSMutableArray *) entityOfType: (NSString *) type;
@end

#endif
